import Foundation

// Interface updated 2026-05-01 — see AGENTS.md
struct TypeDistributionBin: Codable, Identifiable, Hashable, Sendable {
    var id: UUID = UUID()
    var bucketStart: Date = Date.distantPast
    var bucketEnd: Date = Date.distantPast
    var count: Int = 0
    var contentHash: String = ""
    var anchorData: Data?
    init(bucketStart: Date, bucketEnd: Date, count: Int) {
        self.id = UUID()
        self.bucketStart = bucketStart
        self.bucketEnd = bucketEnd
        self.count = count
    }
}
